All of lore.kernel.org
 help / color / mirror / Atom feed
From: Marc Zyngier <maz@kernel.org>
To: linux-kernel@vger.kernel.org,
	linux-arm-kernel@lists.infradead.org, linux-acpi@vger.kernel.org
Cc: Thomas Gleixner <tglx@linutronix.de>,
	Mark Rutland <mark.rutland@arm.com>,
	Will Deacon <will@kernel.org>,
	"Rafael J. Wysocki" <rafael@kernel.org>,
	Rob Herring <robh@kernel.org>,
	Saravana Kannan <saravanak@google.com>,
	Greg Kroah-Hartman <gregkh@linuxfoundation.org>,
	Sven Peter <sven@kernel.org>, Janne Grunau <j@jannau.net>,
	Suzuki K Poulose <suzuki.poulose@arm.com>,
	James Clark <james.clark@linaro.org>,
	Jonathan Cameron <jonathan.cameron@huawei.com>,
	Jinjie Ruan <ruanjinjie@huawei.com>,
	Alexandru Elisei <alexandru.elisei@arm.com>
Subject: [PATCH v4 04/26] platform: Add firmware-agnostic irq and affinity retrieval interface
Date: Mon, 20 Oct 2025 13:29:21 +0100	[thread overview]
Message-ID: <20251020122944.3074811-5-maz@kernel.org> (raw)
In-Reply-To: <20251020122944.3074811-1-maz@kernel.org>

Expand platform_get_irq_optional() to also return an affinity if
available, renaming it to platform_get_irq_affinity() in the
process.

platform_get_irq_optional() is preserved with its current semantics
by calling into the new helper with a NULL affinity pointer.

Reviewed-by: Jonathan Cameron <jonathan.cameron@huawei.com>
Tested-by: Will Deacon <will@kernel.org>
Signed-off-by: Marc Zyngier <maz@kernel.org>
---
 drivers/base/platform.c         | 60 +++++++++++++++++++++++++++------
 include/linux/platform_device.h |  2 ++
 2 files changed, 52 insertions(+), 10 deletions(-)

diff --git a/include/linux/platform_device.h b/include/linux/platform_device.h
index 074754c23d330..ad66333ce85ce 100644
--- a/include/linux/platform_device.h
+++ b/include/linux/platform_device.h
@@ -102,6 +102,8 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 
 extern int platform_get_irq(struct platform_device *, unsigned int);
 extern int platform_get_irq_optional(struct platform_device *, unsigned int);
+extern int platform_get_irq_affinity(struct platform_device *, unsigned int,
+				     const struct cpumask **);
 extern int platform_irq_count(struct platform_device *);
 extern int devm_platform_get_irqs_affinity(struct platform_device *dev,
 					   struct irq_affinity *affd,
diff --git a/drivers/base/platform.c b/drivers/base/platform.c
index 09450349cf323..3a058f63ef0d3 100644
--- a/drivers/base/platform.c
+++ b/drivers/base/platform.c
@@ -150,25 +150,37 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 EXPORT_SYMBOL_GPL(devm_platform_ioremap_resource_byname);
 #endif /* CONFIG_HAS_IOMEM */
 
+static const struct cpumask *get_irq_affinity(struct platform_device *dev,
+					      unsigned int num)
+{
+	const struct cpumask *mask = NULL;
+#ifndef CONFIG_SPARC
+	struct fwnode_handle *fwnode = dev_fwnode(&dev->dev);
+
+	if (is_of_node(fwnode))
+		mask = of_irq_get_affinity(to_of_node(fwnode), num);
+	else if (is_acpi_device_node(fwnode))
+		mask = acpi_irq_get_affinity(ACPI_HANDLE_FWNODE(fwnode), num);
+#endif
+
+	return mask ?: cpu_possible_mask;
+}
+
 /**
  * platform_get_irq_optional - get an optional IRQ for a device
  * @dev: platform device
  * @num: IRQ number index
+ * @affinity: optional cpumask pointer to get the affinity of a per-cpu IRQ
  *
  * Gets an IRQ for a platform device. Device drivers should check the return
  * value for errors so as to not pass a negative integer value to the
- * request_irq() APIs. This is the same as platform_get_irq(), except that it
- * does not print an error message if an IRQ can not be obtained.
- *
- * For example::
- *
- *		int irq = platform_get_irq_optional(pdev, 0);
- *		if (irq < 0)
- *			return irq;
+ * request_irq() APIs. Optional affinity information is provided in the
+ * affinity pointer if available, and NULL otherwise.
  *
  * Return: non-zero IRQ number on success, negative error number on failure.
  */
-int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
+int platform_get_irq_affinity(struct platform_device *dev, unsigned int num,
+			      const struct cpumask **affinity)
 {
 	int ret;
 #ifdef CONFIG_SPARC
@@ -236,8 +248,36 @@ int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
 out:
 	if (WARN(!ret, "0 is an invalid IRQ number\n"))
 		return -EINVAL;
+
+	if (ret > 0 && affinity)
+		*affinity = get_irq_affinity(dev, num);
+
 	return ret;
 }
+EXPORT_SYMBOL_GPL(platform_get_irq_affinity);
+
+/**
+ * platform_get_irq_optional - get an optional IRQ for a device
+ * @dev: platform device
+ * @num: IRQ number index
+ *
+ * Gets an IRQ for a platform device. Device drivers should check the return
+ * value for errors so as to not pass a negative integer value to the
+ * request_irq() APIs. This is the same as platform_get_irq(), except that it
+ * does not print an error message if an IRQ can not be obtained.
+ *
+ * For example::
+ *
+ *		int irq = platform_get_irq_optional(pdev, 0);
+ *		if (irq < 0)
+ *			return irq;
+ *
+ * Return: non-zero IRQ number on success, negative error number on failure.
+ */
+int platform_get_irq_optional(struct platform_device *dev, unsigned int num)
+{
+	return platform_get_irq_affinity(dev, num, NULL);
+}
 EXPORT_SYMBOL_GPL(platform_get_irq_optional);
 
 /**
@@ -150,25 +150,37 @@ devm_platform_ioremap_resource_byname(struct platform_device *pdev,
 /**
- * platform_get_irq_optional - get an optional IRQ for a device
+ * platform_get_irq_affinity - get an optional IRQ and its affinity for a device
  * @dev: platform device
  * @num: IRQ number index
-- 
2.47.3


  parent reply	other threads:[~2025-10-20 12:30 UTC|newest]

Thread overview: 73+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-10-20 12:29 [PATCH v4 00/26] genirq: Add support for percpu_devid IRQ affinity Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 01/26] irqdomain: Add firmware info reporting interface Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 02/26] ACPI: irq: Add IRQ affinity " Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] ACPI: irq: Add interrupt " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 03/26] of/irq: Add IRQ " Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] of/irq: Add interrupt " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` Marc Zyngier [this message]
2025-10-27 16:31   ` [tip: irq/core] platform: Add firmware-agnostic irq and affinity retrieval interface tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 05/26] irqchip/gic-v3: Add FW info retrieval support Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 16:54   ` [PATCH v4 05/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 06/26] irqchip/apple-aic: " Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 07/26] coresight: trbe: Convert to new IRQ affinity retrieval API Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] coresight: trbe: Convert to the new interrupt " tip-bot2 for Marc Zyngier
2025-11-12 16:58   ` [PATCH v4 07/26] coresight: trbe: Convert to new IRQ " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 08/26] perf: arm_pmu: " Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] perf: arm_pmu: Convert to the new interrupt " tip-bot2 for Marc Zyngier
2025-11-12 16:59   ` [PATCH v4 08/26] perf: arm_pmu: Convert to new IRQ " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 09/26] perf: arm_spe_pmu: " Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] perf: arm_spe_pmu: Convert to new interrupt " tip-bot2 for Marc Zyngier
2025-11-12 17:58   ` [PATCH v4 09/26] perf: arm_spe_pmu: Convert to new IRQ " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 10/26] irqchip/gic-v3: Switch high priority PPIs over to handle_percpu_devid_irq() Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:06   ` [PATCH v4 10/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 11/26] genirq: Kill handle_percpu_devid_fasteoi_nmi() Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:07   ` [PATCH v4 11/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 12/26] genirq: Merge irqaction::{dev_id,percpu_dev_id} Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:09   ` [PATCH v4 12/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 13/26] genirq: Factor-in percpu irqaction creation Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 14/26] genirq: Add affinity to percpu_devid interrupt requests Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:13   ` [PATCH v4 14/26] " Jonathan Cameron
2025-11-13 13:37     ` Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 15/26] genirq: Update request_percpu_nmi() to take an affinity Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:15   ` [PATCH v4 15/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 16/26] genirq: Allow per-cpu interrupt sharing for non-overlapping affinities Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:25   ` [PATCH v4 16/26] " Jonathan Cameron
2025-12-04 10:56   ` Daniel Thompson
2025-12-04 14:21     ` Marc Zyngier
2025-12-05  9:19       ` Daniel Thompson
2025-10-20 12:29 ` [PATCH v4 17/26] genirq: Add request_percpu_irq_affinity() helper Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:27   ` [PATCH v4 17/26] " Jonathan Cameron
2025-11-13 13:39     ` Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 18/26] perf: arm_pmu: Request specific affinities for percpu NMI/IRQ Marc Zyngier
2025-10-27 16:31   ` [tip: irq/core] perf: arm_pmu: Request specific affinities for per CPU NMIs/interrupts tip-bot2 for Will Deacon
2025-11-12 18:31   ` [PATCH v4 18/26] perf: arm_pmu: Request specific affinities for percpu NMI/IRQ Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 19/26] perf: arm_spe_pmu: Request specific affinities for percpu IRQ Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] perf: arm_spe_pmu: Request specific affinities for per CPU interrupts tip-bot2 for Marc Zyngier
2025-11-12 18:31   ` [PATCH v4 19/26] perf: arm_spe_pmu: Request specific affinities for percpu IRQ Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 20/26] coresight: trbe: " Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] coresight: trbe: Request specific affinities for per CPU interrupts tip-bot2 for Marc Zyngier
2025-11-12 18:32   ` [PATCH v4 20/26] coresight: trbe: Request specific affinities for percpu IRQ Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 21/26] irqchip/gic-v3: Drop support for custom PPI partitions Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 22/26] irqchip/apple-aic: Drop support for custom PMU irq partitions Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 23/26] irqchip: Kill irq-partition-percpu Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 24/26] genirq: Kill irq_{g,s}et_percpu_devid_partition() Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-10-20 12:29 ` [PATCH v4 25/26] irqdomain: Kill of_node_to_fwnode() helper Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier
2025-11-12 18:35   ` [PATCH v4 25/26] " Jonathan Cameron
2025-10-20 12:29 ` [PATCH v4 26/26] perf: arm_pmu: Kill last use of per-CPU cpu_armpmu pointer Marc Zyngier
2025-10-27 16:30   ` [tip: irq/core] " tip-bot2 for Marc Zyngier

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20251020122944.3074811-5-maz@kernel.org \
    --to=maz@kernel.org \
    --cc=alexandru.elisei@arm.com \
    --cc=gregkh@linuxfoundation.org \
    --cc=j@jannau.net \
    --cc=james.clark@linaro.org \
    --cc=jonathan.cameron@huawei.com \
    --cc=linux-acpi@vger.kernel.org \
    --cc=linux-arm-kernel@lists.infradead.org \
    --cc=linux-kernel@vger.kernel.org \
    --cc=mark.rutland@arm.com \
    --cc=rafael@kernel.org \
    --cc=robh@kernel.org \
    --cc=ruanjinjie@huawei.com \
    --cc=saravanak@google.com \
    --cc=suzuki.poulose@arm.com \
    --cc=sven@kernel.org \
    --cc=tglx@linutronix.de \
    --cc=will@kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.